use std::path::{Component, Path, PathBuf};

/// Converts a path to its absolute form WITHOUT resolving symlinks.
pub fn into_absolute(
  path: impl AsRef<Path>,
  relative_from: impl AsRef<Path>,
) -> PathBuf {
  let path = path.as_ref();

  let mut new_path = if !path.is_absolute() {
    relative_from.as_ref().to_path_buf()
  } else {
    path.to_path_buf()
  };

  for component in path.components() {
    match component {
      Component::Prefix(_) => unimplemented!("windows lol"),
      Component::RootDir => return path.to_path_buf(),
      Component::CurDir => {}
      Component::ParentDir => {
        new_path.pop();
      }
      Component::Normal(s) => {
        new_path.push(s);
      }
    }
  }

  new_path
}
